<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\ParameterBag;

class Mym00104CardapioPesquisaController  extends \Teknisa\GER\Controllers\TeknisaCrud {
	
	protected $connection;
    protected $dataSourceName;
	
	public function __construct($crudService, ParameterBag $parameterBag, $connection) {
		parent::__construct($crudService, $parameterBag);

	    $this->connection = $connection;
	}		
	
	public function altera_Mes(Request $request, Response $response) {
	    $row = $request->getRow();
	    if(isset($row['alteraExibMes']) && $row['alteraExibMes'] == 'S') {
	        $this->altera_ExibMes($request, $response);
	    }
	    if(isset($row['alteraReservaMes']) && $row['alteraReservaMes'] == 'S') {
	        $this->altera_ReservaMes($request, $response);
	    }
	}
	
	public function altera_ExibMes(Request $request, Response $response) {
		try {
			$row = $request->getRow();

    		$query = "SELECT CDSERVFILI, DTMESCARD " 
                    ."FROM CARDADIA " 
                    ."WHERE NRORG = :P_NRORG AND "
                          ."CDFILIAL = :P_CDFILIAL AND "
                          ."NRCARDAPIO = :P_NRCARDAPIO ";
    		$queryParams = [
    			"P_NRORG"	       => $row["NRORG"],
    			"P_CDFILIAL"   => $row["CDFILIAL"],
    			"P_NRCARDAPIO" => $row["NRCARDAPIO"]
    		];
    		
    		$statementQuery = $this->connection->prepare($query);
    		$statementQuery->execute($queryParams);
    		$resultSetQuery = $statementQuery->fetchAll();
		    
		
		    $cdservfili = isset($resultSetQuery[0]["CDSERVFILI"]) ? $resultSetQuery[0]["CDSERVFILI"] : NULL;
		    $dtmescard = isset($resultSetQuery[0]["DTMESCARD"]) ? $resultSetQuery[0]["DTMESCARD"] : NULL;
		    
    		$updComp = "UPDATE COMPCARDPESQ "
                      ."SET IDEXIBEPRATOPESQ = :P_EXIBE "
                      ."WHERE NRORG = :P_NRORG AND "
                            ."CDFILIAL = :P_CDFILIAL AND "
                            ."NRSEQPRATCAR = :P_NRSEQPRATCAR AND "
                            ."CDTIPOPRAT = :P_CDTIPOPRAT AND "
                            ."NRCARDAPIO IN (SELECT C.NRCARDAPIO "
                                             ."FROM CARDADIA C "
                                             ."WHERE C.CDFILIAL = :P_CDFILIAL AND "
                                                   ."C.CDSERVFILI = :P_CDSERVFILI AND "
                                                   ."DTMESCARD  = :P_DTMESCARD) ";
    		$updCompParams = [
			    "P_EXIBE"        => $row["IDEXIBEPRATOPESQ"],
    			"P_NRORG"	     => $row["NRORG"],
    			"P_CDFILIAL"     => $row["CDFILIAL"],
    			"P_NRSEQPRATCAR" => $row["NRSEQPRATCAR"],
    			"P_CDTIPOPRAT"   => $row["CDTIPOPRAT"],
    			"P_CDSERVFILI"   => $cdservfili,
    			"P_DTMESCARD"    => $dtmescard
    		];
    		
    		
    		$statementupdComp = $this->connection->prepare($updComp);
    		$statementupdComp->execute($updCompParams);
    		
    		$updCnf = "UPDATE CNFCAMES "
                        ."SET IDIMPCNFMES = :P_EXIBE "
                      ."WHERE NRORG = :P_NRORG AND "
                            ."CDFILIAL = :P_CDFILIAL AND "
                            ."CDSERVFILI = :P_CDSERVFILI AND "
                            ."NRSEQCNFMES = :P_NRSEQPRATCAR AND "
                            ."CDTIPOPRAT  = :P_CDTIPOPRAT AND "
                            ."DTREFECNFMES = :P_DTMESCARD ";
    		$updCnfParams = [
			    "P_EXIBE"        => $row["IDEXIBEPRATOPESQ"],
    			"P_NRORG"	     => $row["NRORG"],
    			"P_CDFILIAL"     => $row["CDFILIAL"],
    			"P_CDSERVFILI"   => $cdservfili,
    			"P_NRSEQPRATCAR" => $row["NRSEQPRATCAR"],
    			"P_CDTIPOPRAT"   => $row["CDTIPOPRAT"],
    			"P_DTMESCARD"    => $dtmescard
    		];
		    
    		$statementupdCnf = $this->connection->prepare($updCnf);
    		$statementupdCnf->execute($updCnfParams);
		    
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
	
	public function altera_ReservaMes(Request $request, Response $response) {
		try {
			$row = $request->getRow();

    		$query = "SELECT CDSERVFILI, DTMESCARD " 
                    ."FROM CARDADIA " 
                    ."WHERE NRORG = :P_NRORG AND "
                          ."CDFILIAL = :P_CDFILIAL AND "
                          ."NRCARDAPIO = :P_NRCARDAPIO ";
    		$queryParams = [
    			"P_NRORG"	       => $row["NRORG"],
    			"P_CDFILIAL"   => $row["CDFILIAL"],
    			"P_NRCARDAPIO" => $row["NRCARDAPIO"]
    		];
    		
    		$statementQuery = $this->connection->prepare($query);
    		$statementQuery->execute($queryParams);
    		$resultSetQuery = $statementQuery->fetchAll();
		    
		
		    $cdservfili = isset($resultSetQuery[0]["CDSERVFILI"]) ? $resultSetQuery[0]["CDSERVFILI"] : NULL;
		    $dtmescard = isset($resultSetQuery[0]["DTMESCARD"]) ? $resultSetQuery[0]["DTMESCARD"] : NULL;
		    
    		$updComp = "UPDATE COMPCARDPESQ "
                      ."SET IDRESERVAMYMENU = :P_RESERVA "
                      ."WHERE NRORG = :P_NRORG AND "
                            ."CDFILIAL = :P_CDFILIAL AND "
                            ."NRSEQPRATCAR = :P_NRSEQPRATCAR AND "
                            ."CDTIPOPRAT = :P_CDTIPOPRAT AND "
                            ."NRCARDAPIO IN (SELECT C.NRCARDAPIO "
                                             ."FROM CARDADIA C "
                                             ."WHERE C.CDFILIAL = :P_CDFILIAL AND "
                                                   ."C.CDSERVFILI = :P_CDSERVFILI AND "
                                                   ."DTMESCARD  = :P_DTMESCARD) ";
    		$updCompParams = [
			    "P_RESERVA"      => $row["IDRESERVAMYMENU"],
    			"P_NRORG"	     => $row["NRORG"],
    			"P_CDFILIAL"     => $row["CDFILIAL"],
    			"P_NRSEQPRATCAR" => $row["NRSEQPRATCAR"],
    			"P_CDTIPOPRAT"   => $row["CDTIPOPRAT"],
    			"P_CDSERVFILI"   => $cdservfili,
    			"P_DTMESCARD"    => $dtmescard
    		];
    		
    		
    		$statementupdComp = $this->connection->prepare($updComp);
    		$statementupdComp->execute($updCompParams);
    		
    		$updCnf = "UPDATE CNFCAMES "
                        ."SET IDRESERVAMYMENU = :P_RESERVA "
                      ."WHERE NRORG = :P_NRORG AND "
                            ."CDFILIAL = :P_CDFILIAL AND "
                            ."CDSERVFILI = :P_CDSERVFILI AND "
                            ."NRSEQCNFMES = :P_NRSEQPRATCAR AND "
                            ."CDTIPOPRAT  = :P_CDTIPOPRAT AND "
                            ."DTREFECNFMES = :P_DTMESCARD ";
    		$updCnfParams = [
                "P_RESERVA"      => $row["IDRESERVAMYMENU"],
                "P_NRORG"        => $row["NRORG"],
                "P_CDFILIAL"     => $row["CDFILIAL"],
                "P_CDSERVFILI"   => $cdservfili,
                "P_NRSEQPRATCAR" => $row["NRSEQPRATCAR"],
                "P_CDTIPOPRAT"   => $row["CDTIPOPRAT"],
                "P_DTMESCARD"    => $dtmescard
    		];
		    
    		$statementupdCnf = $this->connection->prepare($updCnf);
    		$statementupdCnf->execute($updCnfParams);
		    
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
}