<?php
/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 30/04/14
 * Time: 17:23
 * To change this template use File | Settings | File Templates.
 */

namespace Financeiro\Logic\ContasPagar\Cnab\Retorno;

use \Financeiro\Entities\Cnabpag;
use Geral\Util\Ambiente;
use Setforms2\Shared\ORM\DateTime;
use Setforms2\Shared\Registry;
use Financeiro\Entities\Baixapaga;
use Geral\Util\NovoCodigo;
use Fiscal\Entities\Boletpagar;
use Geral\Util\DateUtil;
use Financeiro\Entities\Errcnabbolp;


/**
 * Class OperacoesCnab
 * @package Financeiro\Logic\ContasPagar\Cnab\Retorno
 */
class OperacoesCnab {


    /**
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdtpbaixapag
     * @param \Setforms2\Shared\ORM\DateTime $dtpagamento
     * @param $cdreducont
     * @param $cdoperador
     * @param $dtlancamento
     * @param $nrorg
     * @param $nrbaixapag
     * @param $vrbaixa
     * @param $nrcnabpag
     * @param string $idstatuspag
     * @throws \Exception
     * @internal param $dsbaixapag
     * @internal param string $statuspag
     */
    public static function insereBaixa($cdempresa, $cdfilial, $cdtpbaixapag, DateTime $dtpagamento,
                                        $cdreducont, $cdoperador, $dtlancamento, $nrorg, $nrbaixapag, $vrbaixa,
                                        $nrcnabpag, $idstatuspag = Baixapaga::STATUS_PENDENTE_CNAB ){

        try {
            $baixapaga = new Baixapaga();
            $baixapaga->setNrbaixapag($nrbaixapag);
            $baixapaga->setVrbaixapag($vrbaixa);
            $baixapaga->setCdempresa($cdempresa);
            $baixapaga->setCdfilial($cdfilial);
            $baixapaga->setIdstatusbpag($idstatuspag);
            $baixapaga->setCdtpbaixapag($cdtpbaixapag);
            $baixapaga->setDtbaixapag($dtpagamento);
            $baixapaga->setCdreducont($cdreducont);
            $baixapaga->setCdoperador($cdoperador);
            $baixapaga->setDtlancbaixap($dtlancamento);
            $baixapaga->setNrorg($nrorg);
            $baixapaga->setDsbaixapag("Baixa realizada via importação CNAB. Retorno nº {$nrcnabpag}");
            $baixapaga->setIdstemicheq('N');
            $baixapaga->setDtinclusao(DateUtil::getDataAtual());
            $baixapaga->setDtultatu(DateUtil::getDataAtual());
            $baixapaga->setCdoperinclusao(\Ambiente::getCdOperador());
            $baixapaga->setCdoperultatu(\Ambiente::getCdOperador());
            $baixapaga->setIdativo('S');

            $baixapaga->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $nrorg
     * @param $nrbaixapag
     * @throws \Exception
     */
    public static function baixaBoleto($cdfilial, $nrlanbolpag, $nrorg, $nrbaixapag){


        try {
            /** @var $boletopagar Boletpagar */
            $boletopagar = Boletpagar::findOneByNrlanbolpagAndCdfilialAndNrorg($cdfilial, $nrlanbolpag, $nrorg);


            $boletopagar->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabpag
     * @param $idtipcnabpag
     * @param DateTime $dtcnabpag
     * @param $cdoperador
     * @param $cdbanco
     * @param $dsArquivo
     * @throws \Exception
     * @internal param $caminhoArquivo
     * @return Cnabpag
     */
    public static function insereRemessa($nrorg, $cdempresa, $nrcnabpag, $idtipcnabpag, DateTime $dtcnabpag , $cdoperador,
                                  $cdbanco,$dsArquivo){
        try
        {
            $cnabpag = new Cnabpag();
            $cnabpag->setNrorg($nrorg);
            $cnabpag->setCdempresa($cdempresa);
            $cnabpag->setNrcnabpag($nrcnabpag);
            $cnabpag->setDsarqcnabpag($dsArquivo);
            $cnabpag->setIdtipcnabpag($idtipcnabpag);
            $cnabpag->setDtcnabpag($dtcnabpag);
            $cnabpag->setCdoperador($cdoperador);
            $cnabpag->setCdbanco($cdbanco);
            $cnabpag->setDtinclusao(DateUtil::getDataAtual());

            $cnabpag->persist();
            Registry::getEntityManager()->flush();

            return $cnabpag;
        }catch(\Exception $e)
        {
            throw $e;
        }
    }

    /**
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $vrfinalbpag
     * @param $cdfilibaipag
     * @param $nrbaixapag
     * @param $nrcnabpagret
     * @param $idsitucnabp
     * @param $nrorg
     * @throws \Exception
     */
    public static function alteraBoleto($cdfilial, $nrlanbolpag, $vrfinalbpag, $cdfilibaipag, $nrbaixapag,
                                        $nrcnabpagret, $idsitucnabp, $nrorg){

        try {
            /** @var $boletpagar Boletpagar */
            $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);
            $boletpagar->setVrfinalbpag($vrfinalbpag);
            $boletpagar->setCdfilibaipag($cdfilibaipag);
            $boletpagar->setNrbaixapag($nrbaixapag);
            $boletpagar->setNrcnabpagret($nrcnabpagret);
            $boletpagar->setIdsitucnabp($idsitucnabp);
            $boletpagar->setIdstatbpag(Boletpagar::STATUS_PAGAMENTO_PLANEJADO);
            $boletpagar->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $cdfilial
     * @param $nrbaixapag
     * @param $valor
     * @throws \Exception
     */
    public static function insereValorBaixa($cdfilial, $nrbaixapag, $valor){

        try {
            /** @var $baixapaga Baixapaga */
            $baixapaga = Baixapaga::findOneByNrbaixapagAndCdfilialAndNrorg($nrbaixapag, $cdfilial, Ambiente::getNrorg());
            $baixapaga->setVrbaixapag($baixapaga->getVrbaixapag() + $valor);

            $baixapaga->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdbanco
     * @param $cdercnabbolp
     * @throws \Exception
     */
    public static function insereOcorencia($cdfilial, $nrlanbolpag, $cdbanco, $cdercnabbolp){

        try {
            $errcnabbolp = new Errcnabbolp();
            $errcnabbolp->setCdfilial($cdfilial);
            $errcnabbolp->setNrlanbolpag($nrlanbolpag);
            $errcnabbolp->setNrorg(Ambiente::getNrorg());
            $errcnabbolp->setCdbanco($cdbanco);
            $errcnabbolp->setCdercnabbolp($cdercnabbolp);
            $errcnabbolp->setDtinclusao(DateUtil::getDataAtual());
            $errcnabbolp->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;

        }
    }

    /**
     * @param $nrlanbolpag
     * @param $cdfilial
     * @param $nrorg
     * @throws \Exception
     */
    public static function deletaErroCnab($nrlanbolpag, $cdfilial, $nrorg){

        try{
        /** @var $errcnabbolp Errcnabbolp  */
        $errcnabbolps = Errcnabbolp::findByNrlanbolpagAndCdfilialAndNrorg($nrlanbolpag, $cdfilial, $nrorg);
        foreach($errcnabbolps as $errcnabbolp){
           $errcnabbolp->delete();
           $errcnabbolp->persist();
        }
        Registry::getEntityManager()->flush();
        }
        catch(\Exception $e){
            throw $e;
        }

    }

}