<?php
namespace Fiscal\Logic\NotaFiscal\Lancamento;


use Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use Fiscal\Entities\Imposnftot;
use Fiscal\Entities\IntgNotafiscal;
use Fiscal\Entities\Itemproimpos;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Logic\Impostos\GeradorImposto;
use Fiscal\Logic\Impostos\ValidacaoImpostos;
use Fiscal\Logic\NotaFiscal\ConferenciaErros;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use Fiscal\Logic\NotaFiscal\RateioNotaFiscal;
use Fiscal\Logic\NotaFiscal\ReplicacaoAutomatica;
use Setforms2\Shared\Registry;

/**
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 12/11/12
 * Time: 09:28
 * To change this template use File | Settings | File Templates.
 */
class LancamentoNF
{

    /**
     * Executa o processo completo de lancamento de Nota fiscal.
     *
     * @static
     * @param $cdforneced
     * @param $cdserie
     * @param $nrnotafisc
     * @param $cdtipooperacao
     * @throws \Exception
     * @throws \Geral\Exceptions\Exception
     */
    public static function executaLancamento($cdforneced, $cdserie, $nrnotafisc, $cdtipooperacao)
    {
        try {
            $notaFiscal = self::importadorNotafiscal($cdforneced, $cdserie, $nrnotafisc, $cdtipooperacao);

            if (!$notaFiscal) {
                throw new \Geral\Exceptions\Exception("Nenhuma Nota Fiscal foi gerada.");
            }

            self::fecharNotaFiscal($notaFiscal);
        } catch (\Exception $e) {
            //            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Excecuta o processo de importacao da nota fiscal.
     *
     * @static
     * @param $cdforneced
     * @param $cdserie
     * @param $nrnotafisc
     * @param $cdtipooperacao
     *
     * @return \Fiscal\Entities\Notafiscal|null
     *
     * @throws \Exception
     */
    private static function importadorNotafiscal($cdforneced, $cdserie, $nrnotafisc, $cdtipooperacao)
    {
        try {
            $nrorg         = \Geral\Util\Ambiente::getNrorg();
            $importadorNFE = new \Fiscal\Logic\ImportacaoNFE\ImportadorNFE($nrorg, $cdforneced, $cdserie, $nrnotafisc);
            $importadorNFE->importaDocumento($cdtipooperacao);
            $notaFiscal = $importadorNFE->getNotaFiscal();
            return $notaFiscal;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function fecharNotaFiscal(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManagerConnection();
            $connection->beginTransaction();
            // $vrperc = self::geracaoRateioAnterior($notaFiscal);
            self::geracaoDeImpostos($notaFiscal);
            self::bloqueiaNotaOrigem($notaFiscal);
            self::validarImpostos($notaFiscal);
            //            self::CalcularCustoLiquido($notaFiscal); Não chamar
            self::replicacaoAutomatica($notaFiscal);
            self::atualizaEstoque($notaFiscal);
            self::verificaOrcamento($notaFiscal);
            ProceduresNF::callProcedureBaixaParcial($notaFiscal);
            self::geraRateioPadrao($notaFiscal);
            $connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa as procedures relacionadas a geracao de impostos.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function geracaoDeImpostos(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            // Procedure de impostos temporarios.
            //             GeradorImposto::callProcedureGravaImpostoTemporario($notaFiscal);
            // Geracao de impostos.
            GeradorImposto::callProcedureImposto($notaFiscal);
            Registry::getEntityManager()->refresh($notaFiscal);
            ConferenciaErros::confereErros($notaFiscal);
            // Procedure GRAVA_IMPOSTO.
            GeradorImposto::callProcedureGravaImposto($notaFiscal);
            // Conferencia de erros da NF.
            //             ConferenciaErros::executaProcedureConferenciaDeErros($notaFiscal);
            //			Registry::getEntityManager()->refresh($notaFiscal);
            //            ConferenciaErros::confereErros($notaFiscal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Bloqueia as notas fiscais de origem.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    private static function bloqueiaNotaOrigem(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {

            $itensNF = \Geral\Entities\Itemnotafisc::findByCdfilialAndNrlanctonfAndNrorg(
                $notaFiscal->getFilial()->getCdfilial(),
                $notaFiscal->getNrlanctonf(),
                \Geral\Util\Ambiente::getNrorg()
            );

            /** @var $itemnotafisc \Geral\Entities\Itemnotafisc */
            foreach ($itensNF as $itemnotafisc) {
                $nf = \Fiscal\Entities\Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg(
                    $itemnotafisc->getLanctonfit()->getFilial()->getCdfilial(),
                    $itemnotafisc->getLanctonfit()->getNrlanctonf(),
                    \Geral\Util\Ambiente::getNrorg()
                );

                if ($nf->getEntrsaidop()->getCdtipooper() === \Geral\Entities\Tipooperac::OPERACAO_ENTRADA) {
                    $nf->setIdbloqueado(\Fiscal\Entities\Notafiscal::NOTA_FISCAL_BLOQUEADA);
                    $nf->persist();
                }
            }

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            echo $e->getMessage();
            throw $e;
        }
    }

    /**
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function validarImpostos(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            $validacaoImpostos = new ValidacaoImpostos();
            $validacaoImpostos->validaImpostosObrigatorios($notaFiscal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa a replicacao de documentos, caso este esteja parametrizado para tal.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     * @throws \Exception
     */
    private static function replicacaoAutomatica(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        $replicacaoAutomatica = new ReplicacaoAutomatica($notafiscal);
        if ($replicacaoAutomatica->permiteReplicacaoAutomatica()) {
            try {
                $replicacaoAutomatica->geraReplicacaoAutomatica();
            } catch (\Exception $e) {
                throw $e;
            }
        }
    }

    /**
     * Executa a atualizacao de estoque e a verificacao do estoque, se este foi devidamento atualizado.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    private static function atualizaEstoque(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        /** @var $notasFiscais array Acumula todas as notas, a nota fiscal de origem e as replicas. */
        $notasFiscais   = \Fiscal\Entities\Notafiscal::findByIddocreplicAndNrorg(
            $notaFiscal->getFilial()->getCdfilial() . $notaFiscal->getNrlanctonf(),
            \Geral\Util\Ambiente::getNrorg()
        );
        $notasFiscais[] = $notaFiscal;

        foreach ($notasFiscais as $NF) {
            self::executaAtualizacaoEstoque($NF);
        }
        //        /** @var $lanctoestoq \Estoque\Entities\Lanctoestoq */
        //        $lanctoestoq = \Estoque\Entities\Lanctoestoq::findOneBycdfilialAndnrlanctonfAndNrorg(
        //            $notaFiscal->getFilial()->getCdfilial(), $notaFiscal->getNrlanctonf(), \Geral\Util\Ambiente::getNrorg()
        //        );
        //
        //        if ($lanctoestoq !== null && AtualizadorDeEstoque::autorizaAtualzacaoDeEstoque($lanctoestoq)) {
        //            self::atualizarEstoqueReplicas($notaFiscal);
        //        }
    }

    /**
     *
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    private static function executaAtualizacaoEstoque(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        /** @var $filial \Geral\Entities\Filial */
        $filial = null;
        if ($notaFiscal->getEntrsaidop()->getIdentrsaidop() == \Geral\Entities\Tipooperac::OPERACAO_ENTRADA) {
            $filial = $notaFiscal->getFilirece();
        } else {
            $filial = $notaFiscal->getFiliemit();
        }

        try {
            /** @var $lanctoestoq \Estoque\Entities\Lanctoestoq */
            $lanctoestoq = \Estoque\Entities\Lanctoestoq::findOneBycdfilialAndnrlanctonfAndNrorg(
                $filial->getCdfilial(), $notaFiscal->getNrlanctonf(), \Geral\Util\Ambiente::getNrorg()
            );

            if ($lanctoestoq !== null && AtualizadorDeEstoque::autorizaAtualizacaoDeEstoque($lanctoestoq)) {
                self::CalcularCustoLiquido($notaFiscal);
                AtualizadorDeEstoque::atualizarEstoque($lanctoestoq);
                AtualizadorDeEstoque::verificarLancamentoEstoqueDaNota($notaFiscal);
                AtualizadorDeEstoque::atualizaOperadorLancamentoEstoque($lanctoestoq);
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Excecuta a procedure de CUSTO_LIQUIDO.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public static function calcularCustoLiquido(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        $lanctoestoq = \Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg(
            $notaFiscal->getFilial()->getCdfilial(),
            $notaFiscal->getNrlanctonf(),
            \Geral\Util\Ambiente::getNrorg()
        );

        \Estoque\Logic\AtualizacaoEstoque\CalculaCustoLiquido::CalcularCustoLiquido(
            $lanctoestoq->getCdfilial(),
            $lanctoestoq->getNrlancestq(),
            $notaFiscal->getVrprodutos()
        );
    }

    /**
     * Executa o processo de verificacao de orcamento. Caso os valores produzidos pela nota superem o valor total
     * orcado, este metodo lanca execao, informando o erro e parando a execucao. Este processo tambem pode produzir
     * mensagens de alerta, caso o valor da nota supere o valor parametrizado para alerta de valor de orcamento.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    private static function verificaOrcamento(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            $verificadorDeOrcamento = new \OrcamentoOperacional\Logic\LiberacaoOrcamento($notaFiscal);
            $verificadorDeOrcamento->verificaOrcamento();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Geracao de rateio contabil.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function geraRateioPadrao(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            $geradorDeRateio = new RateioNotaFiscal($notaFiscal);
            $geradorDeRateio->excluirRateio();
            $geradorDeRateio->geraRateioContabil();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta todos os dados do lancamento das tabelas temporarias
     * TRABITPROIMPOS e TRABIMPOSNFTOT
     * @param Notafiscal $notafiscal
     * @return void
     */
    public static function deletaTabelasTemporariasImpostos(Notafiscal $notafiscal)
    {
        /** @var $itproimpos Trabitproimpos */
        $trabitproimpos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrorg(
            $notafiscal->getCdfilial(),
            $notafiscal->getNrlanctonf(),
            $notafiscal->getNrorg()
        );

        foreach ($trabitproimpos as $itproimpos) {
            $itproimpos->delete();
        }

        $trabimposnftot = Trabimposnftot::findByCdfilimptotAndNrlancimptotAndNrorg(
            $notafiscal->getCdfilial(),
            $notafiscal->getNrlanctonf(),
            $notafiscal->getNrorg()
        );

        /** @var $imposnftot Trabimposnftot */
        foreach ($trabimposnftot as $imposnftot) {
            $imposnftot->delete();
        }

        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    /**
     * Preenche as tabelas temporarias de imposto TRABITPROIMPOS E TRABIMPOSNFTOT
     * para a execucao correta da procedure IMPOSTO_ENTRADA no fechamento
     * @param Notafiscal $notafiscal
     * @return void
     */
    public static function preencheTabelasTemporariasImpostos(Notafiscal $notafiscal)
    {
        $itemproimpos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrorg(
            $notafiscal->getCdfilial(),
            $notafiscal->getNrlanctonf(),
            $notafiscal->getNrorg()
        );

        /** @var $imposto_item Itemproimpos */
        foreach ($itemproimpos as $imposto_item) {
            $trabitproimpos = new Trabitproimpos();

            $trabitproimpos->setCdfilimpoit($imposto_item->getCdfilial());
            $trabitproimpos->setNrlancimpoit($imposto_item->getNrlanctonf());
            $trabitproimpos->setNrseqimpoit($imposto_item->getNrseqitprod());
            $trabitproimpos->setCdimpostoit($imposto_item->getCdimposto());
            $trabitproimpos->setNrseqitpro('1');
            $trabitproimpos->setNrorg($imposto_item->getNrorg());

            $trabitproimpos->setCdoperinclusao($imposto_item->getCdoperinclusao());
            $trabitproimpos->setDtinclusao($imposto_item->getDtinclusao());

            $trabitproimpos->setVrcontimpoit('0');
            $trabitproimpos->setVrredbcimpoit('0');
            $trabitproimpos->setVrbcalcimpoit('0');
            $trabitproimpos->setVrpdedimpoit('0');

            $trabitproimpos->setIdimpdesotrabit('N');
            $trabitproimpos->setIdmotdesotrabit('9');
            $trabitproimpos->setIddifaliqcuspti('N');

            $trabitproimpos->setVrimposit($imposto_item->getVrimpostoit());
            $trabitproimpos->setVraliqimpoit($imposto_item->getVraliqitem());
            $trabitproimpos->setIdvalorinfo($imposto_item->getIdvalorinfit());
            $trabitproimpos->setCdoperultatu($imposto_item->getCdoperultatu());
            $trabitproimpos->setDtultatu($imposto_item->getDtultatu());

            $trabitproimpos->persist();
        }

        $imposnftot = Imposnftot::findByCdfilialAndNrlanctonfAndNrorg(
            $notafiscal->getCdfilial(),
            $notafiscal->getNrlanctonf(),
            $notafiscal->getNrorg()
        );

        /** @var $imposto_nota Imposnftot */
        foreach ($imposnftot as $imposto_nota) {
            $trabimposnftot = new Trabimposnftot();

            $trabimposnftot->setCdfilimptot($imposto_nota->getCdfilial());
            $trabimposnftot->setNrlancimptot($imposto_nota->getNrlanctonf());
            $trabimposnftot->setCdimpostot($imposto_nota->getCdimposto());
            $trabimposnftot->setVrimpostot($imposto_nota->getVrtotimponf());
            $trabimposnftot->setVrbcalimptot('0');
            $trabimposnftot->setNrorg($imposto_nota->getNrorg());
            $trabimposnftot->setDtinclusao($imposto_nota->getDtinclusao());
            $trabimposnftot->setDtultatu($imposto_nota->getDtultatu());
            $trabimposnftot->setCdoperinclusao($imposto_nota->getCdoperinclusao());
            $trabimposnftot->setCdoperultatu($imposto_nota->getCdoperultatu());
            $trabimposnftot->setIddifaliqcusptn('N');

            $trabimposnftot->persist();
        }

        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    /**
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function fecharNotaFiscalEntrada(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            //            $connection = \Setforms2\Shared\Registry::getEntityManagerConnection();
            //            $connection->beginTransaction();
            // $vrperc = self::geracaoRateioAnterior($notaFiscal);
            self::geracaoDeImpostos($notaFiscal);
            self::bloqueiaNotaOrigem($notaFiscal);
            self::validarImpostos($notaFiscal);
            //            self::CalcularCustoLiquido($notaFiscal); Não chamar
            //            self::replicacaoAutomatica($notaFiscal);
            self::atualizaEstoque($notaFiscal);
            //            self::verificaOrcamento($notaFiscal);
            ProceduresNF::callProcedureBaixaParcial($notaFiscal);
            self::atualizaXMLcadastrado($notaFiscal);
            self::geraRateioPadrao($notaFiscal);
            //            self::atualizaStatusNF($notaFiscal, Notafiscal::PENDENTE);
            Registry::getEntityManager()->flush();
            //            $connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaXMLcadastrado(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        /** @var $xml IntgNotafiscal */
        $xml = IntgNotafiscal::findOneByNracessonfe($notafiscal->getNracessonfe());

        if ($xml instanceof IntgNotafiscal) {
            $xml->setIdstatusnfe('U');
            $xml->setIdimportada('S');
            $xml->setCdfilirece($notafiscal->getCdfilial());
            $xml->setDtutilizaxml($notafiscal->getDtinclusao());
            $xml->setCdfilial($notafiscal->getCdfilial());
            $xml->setNrlanctonf($notafiscal->getNrlanctonf());

            $xml->persist();
        }
    }

    public static function atualizaStatusNF(Notafiscal $notafiscal, $status)
    {
        $notafiscal->setIdstatus($status);
        $notafiscal->persist();
        Registry::getEntityManager()->flush();
    }

    /**
     * Geracao de rateio anterior.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @return array
     */
    public static function geracaoRateioAnterior(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        $rateio = new RateioNotaFiscal($notaFiscal);
        return $rateio->geraRateioAnterior();
    }


}