<?php
namespace Importacao\Logic\Integracao;

use Geral\Entities\Itemprojimport;
use Geral\Entities\Logprojimport;
use Importacao\Logic\Bases\ModeloImportacao\EmpilhaDados;

class ProjetoImportacao
{
    const ESTRUTURA = 1;
    const OCUPACAO = 2;
    const ESCALA_DE_TRABALHO = 3;
    const VINCULO = 4;
    const DEPENDENTE = 5;
    const ALTERACAO_SALARIAL = 6;
    const ALTERACAO_ESCALA = 7;
    const ALTERACAO_OCUPACAO = 8;
    const SITUACAO_FUNCIONAL = 9;
    const FERIAS = 10;
    const EVENTO = 11;
    const MOVIMENTACOES_ESTRUTURAS = 12;
    const MOVIMENTOS_FINANCEIROS = 13;
    const NRORG_PADRAO = 0;

    const CARACTER_HASHTAG = '#';
    const MSG_SUCESSO = 'Registro inserido com sucesso';

    const CDOPERADOR = '9999';
    const NRORG_INCLUSAO = '1';

	private $tabprojimport;
    private $nrorg;
    private $itemprojimport;
    private static $instancia;
    private $conn;

	public function __construct(Itemprojimport $itemprojimport = null, $nrorg = null)
	{
        $this->itemprojimport = $itemprojimport;
        $this->nrorg          = $nrorg;
        $nrorg                = self::NRORG_PADRAO;

        if ($itemprojimport instanceof Itemprojimport) {
            $nrprojimport = $itemprojimport->getNrprojimport();
            $this->tabprojimport = \Geral\Entities\Tabprojimport::findOneByNrprojimportAndNrorg($nrprojimport, $nrorg);
        }

        self::$instancia = $this;
        $this->conn = self::getConnection();
	}

    /**
     * @return ProjetoImportacao
     */
    public static function getInstancia() {
        return self::$instancia;
    }

	public function concluir() 
	{
		try {
			$this->tabprojimport->setIdprojimport(\Geral\Entities\Tabprojimport::STATUS_CONCLUIDO);
			$this->tabprojimport->persist();
			\Setforms2\Shared\Registry::getEntityManager()->flush();
		} catch (\Exception $e) {
			throw $e;
		}
	}

    /**
     * @throws \Exception
     */
    public function gerarSQLImportacao()
    {
        set_time_limit(0);
        
        $agrupamento = $this->getDsTipoTransferencia();
        $xlsPath     = $this->itemprojimport->getNmarqimport();
        $nrorg       = $this->nrorg;

        $dir_data          = __DIR__."/../../../../../../../mobile/files/uploads/".$nrorg."/data/";
        $dir_results       = __DIR__."/../../../../../../../mobile/files/uploads/".$nrorg."/results/";
        $dir_helperJson    = __DIR__."/../../../../../../src/PortalMigracao/Migracao/migracao/helper.json";
        $dir_estruturaJson = __DIR__."/../../../../../../src/PortalMigracao/Migracao/migracao/estrutura.json";

        $xlsFilePath  = realpath($dir_data.$xlsPath);

        if (!file_exists($xlsFilePath)) {
            throw new \Exception("Arquivo não encontrado");
        }

        $xls          = new \Importacao\Logic\Util\ExcelUtils($xlsFilePath);
        $data         = $xls->buildDataIntoJson();

        $helpers      = json_decode(file_get_contents($dir_helperJson), true);
        $estrutura    = json_decode(file_get_contents($dir_estruturaJson), true);

        // Construindo o objeto de relacao entre dados e modelo (tabelas sql).
        $dataBuilder  = new \Importacao\Logic\Bases\ModeloImportacao\DataBuilder($estrutura, $data, $helpers, $nrorg);
        $dataBuilder->addAgrupador($agrupamento);
        $data         = $dataBuilder->build();

        // Construindo as queries.
        $queryBuilder = new \Importacao\Logic\Bases\ModeloImportacao\Querybuilder($data);
        $date         = new \DateTime();

        // Escreve o arquivo .sql
        file_put_contents($dir_results.$agrupamento.".sql", $queryBuilder->buildGrupos());

        //$result = "Arquivo $agrupamento.sql gerado com sucesso.";

    }

    /**
     * @throws \Exception
     */
    public function importarArquivoToEntities()
    {

        $agrupamento = $this->getDsTipoTransferencia();
        $arquivo     = $this->itemprojimport->getNmarqimport();
        $tipoArquivo = explode('.', $arquivo);
        $tipoArquivo = end($tipoArquivo);
        $nrorg       = $this->nrorg;

        $dir_estruturaJson = __DIR__."/../../../../../../src/PortalMigracao/Migracao/migracao/estrutura.json";
        $dir_data          = __DIR__."/../../../../../../../mobile/files/uploads/".$nrorg."/data/";
        $filePath          = realpath($dir_data.$arquivo);

        if (!file_exists($filePath)) {
            throw new \Exception("Arquivo não encontrado");
        }

        if ($tipoArquivo == 'xls' || $tipoArquivo == 'xlsx') {

            $xls  = new \Importacao\Logic\Util\ExcelUtils($filePath);
            $data = $xls->buildDataIntoJson();

        } elseif ($tipoArquivo == 'csv') {

            $delimitador = "|";
            $cerca       = '"';
            $csv         = new \Importacao\Logic\Util\CSVUtils($filePath, $delimitador, $cerca);
            $data        = $csv->buildDataIntoJson();

        } else {
            throw new \Exception("Tipo de arquivo ". $tipoArquivo ." não suportado.");
        }

        $estruturaJson = json_decode(file_get_contents($dir_estruturaJson), true);        

        // valida se as colunas do excel existem no agrupamento do estrutura.json
        $this->validaColunasExcel($data, $estruturaJson, $agrupamento);

        // empilha os dados na filha do Activemq
        $empilhador = new EmpilhaDados($data, $this->itemprojimport->getNritemprojimp(), $nrorg, $agrupamento, $nrorg);
        $empilhador->exec();
    }

    /**
     * @param $novaetapavalidacao
     */
    public function atualizaPercentStatus($novaetapavalidacao)
    {
        $this->itemprojimport->setIdtpvalidacao($novaetapavalidacao);
        $this->itemprojimport->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    /**
     * @return mixed
     */
    public function getDsTipoTransferencia()
    {
        return $this->tabprojimport->getDsprojimport();
    }

    /**
     * @param null $nritemprojimport
     * @param $mensagem
     * @param null $nrlinha
     * @param null $dslinha
     * @param null $nrtpvalidacao
     * @throws \Exception
     */
    public function criaLogDeItemProjeto($nritemprojimport = null, $mensagem, $nrlinha = null, $dslinha = null, $nrtpvalidacao = null)
    {
        try {

            if (empty($nritemprojimport)) {
                $nritemprojimport = $this->itemprojimport->getNritemprojimp();
            }

            $logprojimport = new Logprojimport();        
            $logprojimport->setNrlogprojimport($logprojimport->getMaxNrlogprojimport($this->nrorg));
            $logprojimport->setNrorg($this->nrorg);
            $logprojimport->setNritemprojimp($nritemprojimport);
            $logprojimport->setDscomandoimport($mensagem);
            $logprojimport->setNrlinharqimport($nrlinha);
            $logprojimport->setDslinharqimport($dslinha);
            $logprojimport->setNrtpvalidacao($nrtpvalidacao);
            $logprojimport->setDtinclusao(\Geral\Util\DateUtil::getDataAtual());

            $logprojimport->persist();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @throws \Exception
     */
    public function executarSQLImportacao()
    {
        // define caminho do arquivo
        $xlsPath     = $this->getDsTipoTransferencia().".sql";
        $nrorg       = $this->nrorg;
        $dir_results = __DIR__."/../../../../../projects/importacao/files/upload/".$nrorg."/results/";

        // retorna o caminho absoluto do arquivo
        $xlsFilePath  = realpath($dir_results.$xlsPath);

        //  verfica se o arquivo existe
        if (!file_exists($xlsFilePath)) {
            throw new \Exception("Arquivo '$xlsPath' não encontrado.");
        }

        // importa o arquivo SQL para um array
        $content = file($xlsFilePath);

        if (!sizeof($content)) {
            throw new \Exception("Nenhuma instrução SQL encontrada.");
        }

        // percorre as linhas
        foreach ($content as $key => $line) {

            $line = str_replace(';', '', $line);
            $line = trim($line);

            // verifica se for linha em branco ou linha com comentário pula a mesma
            if (empty($line) || substr($line, 0, 1) == self::CARACTER_HASHTAG) {
                continue;
            }

            // executa os comandos de cada linha do arquivo
            try {
                $this->executeSQL($line);
                // criar log de sucesso
                $this->criaLogDeItemProjeto($this->itemprojimport->getNritemprojimp(), self::MSG_SUCESSO, $key, $line);
            }  catch (\Exception $e) {
                // cira log de exceção com erro
                $this->criaLogDeItemProjeto($this->itemprojimport->getNritemprojimp(), $e->getMessage(), $key, $line);
            }

        }

        $result = "Arquivo $xlsPath executado com sucesso, favor verificar os LOGS.";
        tk_message($result, 'success', 5000);
    }

    /**
     *
     */
    public function atualizaItemArqImportado($status)
    {
        $this->itemprojimport->setIdarqimportado(Itemprojimport::ARQ_IMP_SIM);
        $this->itemprojimport->setIdprojimport($status);
        $this->itemprojimport->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    /**
     * @throws \Exception
     */
    public function limpaEntitiesMigByItemprojimport()
    {
        $dir_estruturaJson =__DIR__."/../../../../../../src/PortalMigracao/Migracao/migracao/estrutura.json";
        $estrutura         = json_decode(file_get_contents($dir_estruturaJson), true);
        $nrprojimport      = $this->itemprojimport->getNrprojimport();

        $tables = array_keys($estrutura[$this->getDsTipoTransferencia()]);

        foreach ($tables as $tablename) {
            try {

                $classParts = explode("_", $tablename);
                $tabela     = '';

                foreach ($classParts as $part) {
                    $part    = ($part == "GPE" || $part == "FPA") ? null : $part;
                    $tabela .= $part;
                }

                $tablename = 'MIG_'.$tabela;

                $sql = "DELETE FROM $tablename WHERE NRORG = $this->nrorg AND NRPROJIMPORT = $nrprojimport";
                \Setforms2\Shared\Registry::getEntityManagerConnection()->exec($sql);

            } catch (\Exception $e) {
                throw $e;
            }
        }
    }

    /**
     *
     */
    public function migracaoTabelasDefinitivas()
    {
        $agrupamento = $this->getDsTipoTransferencia();
        $nrorg       = $this->nrorg;

        $dir_estruturaJson =__DIR__."/../../../../../projects/importacao/services/migracao/estrutura.json";
        $estruturaJson     = json_decode(file_get_contents($dir_estruturaJson), true);

        // Construindo o objeto de relacao entre dados e modelo (tabelas sql).
        $tabelasDefinitivas  = new \Importacao\Logic\Bases\ModeloImportacao\MigraTabelasDefinitivas($estruturaJson);
        $tabelasDefinitivas->addAgrupador($agrupamento);
        $data         = $tabelasDefinitivas->build();
    }

    /**
     * Valida de o nome das colunas no excel existem no agrupamento do estrutura.json
     *
     * @param $data
     * @param $estruturaJson
     * @param $agrupamento
     * @throws \Exception
     */
    private function validaColunasExcel($data, $estruturaJson, $agrupamento)
    {
        $colunas = array();

        foreach ($estruturaJson as $nomeAgrupamento => $tabelas) {
            if ($agrupamento === $nomeAgrupamento) {
                foreach ($tabelas as $nomeTabela => $dadosTabela) {
                    foreach ($dadosTabela as $nomeColuna => $dadosColuna) {
                        $colunas[] = $nomeColuna;
                        $colunas[] = $dadosColuna;
                    }
                }
            }
        }

        $count = 0;
        foreach ($data as $key => $linhaDados) {
            foreach ($linhaDados as $nomeColunaExcel => $valorColunaExcel) {
                foreach ($colunas as $nomeColunaJson) {
                    if (strpos($nomeColunaJson, $nomeColunaExcel) || $nomeColunaJson == $nomeColunaExcel) {
//                        tk_warning("$nomeColunaExcel Existe em $nomeColunaJson");
                        $count++;
                    }
                }

                if ($count == 0) {
                    throw new \Exception("A coluna '$nomeColunaExcel' não existe.");
                }

                $count = 0;
            }
            break;
        }
    }

    /**
     * @param $query
     */
    public function executeSQL($query)
    {
        $this->conn->execute($query);
    }

    public static function getConnection()
    {
        return \Setforms2\Shared\Registry::getEntityManager()->getConnection();
    }

    public function getNrprojimport()
    {
        return $this->itemprojimport->getNrprojimport();
    }

    public static function getNrprojetoimport()
    {
        return self::getInstancia()->getNrprojimport();
    }

    public static function getNrorg() {
        return self::getInstancia()->nrorg;
    }

    public static function getNrorgPadrao() {
        return self::NRORG_PADRAO;
    }
}